#!/bin/bash

verbose=$1

function flatten()
{
    awk '

    $1 ~ /^active_var/ {
        sub(/active_var\[/,"");
        sub(/\]/,"");
        split($0, token, "=");
        active_var[token[1]] = token[2];
        num_active_vars = 1 + num_active_vars;
    }

    $1 ~ /^term:$/ { weight[$2] = 1; digits = length($2); }

    END {

        # for (var_idx in active_var)
        for (i=0; i<num_active_vars; ++i)
            printf("active_var[%s]=%s\n", i, active_var[i]);

	for (i=1; i<=digits; ++i) {

	    for (t in weight) {
		left = substr(t,1,i-1);
		middle = substr(t,i,1);
		right = substr(t,i+1,digits-i);
		if (middle ~ /1|2/) {
		    weight2[t] = weight2[t] + weight[t];
		} else {
		    weight2[left "1" right] = weight2[left "1" right] - weight[t];
		    weight2[left "2" right] = weight2[left "2" right] + weight[t];
		}
	    }

	    delete weight;

	    for (t in weight2) weight[t] = weight2[t];

	    delete weight2;
	}

        printf("\n");

	for (t in weight) if (weight[t]) printf("weight[\"%s\"]=%d\n", t, weight[t]);

        printf("\n");

        expr = "";

	for (term_string in weight) {
            term_weight = weight[term_string];
            if (term_weight == 0) continue;
            term = sprintf("%d", term_weight);
            for (i=1; i<=digits; ++i) {
                c = substr(term_string, i, 1);
                if (c ~ /1/) term = term sprintf("*%s", active_var[digits-i]);
            }
            printf("term: %s\n", term);
            expr = expr " + " term;
        }

        printf("\npenalty: %s\n", expr);
    }'
}

function validate()
{
    local num_active_vars
    num_active_vars=0

    while read
    do
        if [[ $REPLY == active_var* ]]
        then
            active_var[num_active_vars]=${REPLY##*=}
            num_active_vars=$((1+num_active_vars))
        fi

        if [[ $REPLY == penalty* ]]
	then
	    penalty=${REPLY#penalty:}
	fi
    done

    local header
    local format
    local dashes

    header=
    format=
    for ((i=0; i<num_active_vars; ++i))
    do
	header=" ${active_var[i]} ${header}"
	format=" %${#active_var[i]}d ${format}"
        dashes=" $(repeat_char ${#active_var[i]} -) ${dashes}"
    done
    header="${header} value"
    dashes="${dashes} $(repeat_char 5 -)"
    format="${format} %5d\\n"

    echo
    echo "$header"
    echo "$dashes"

    num_states=$((2 ** num_active_vars))
    for ((iloop=0; iloop<num_states; ++iloop))
    do
        state=
        i=$iloop
        for ((j=0; j<num_active_vars; ++j))
        do
            lsb=$(( i % 2 ))
            i=$(( (i-lsb)/2 ))
            state="${lsb} ${state}"
            eval ${active_var[j]}='$lsb'
        done

        val=$(( penalty ))
	printf "${format}" ${state} $val
    done
}

function repeat_char
{
    local r
    local c
    local i
    local s
    r=$1
    c=$2
    for ((i=0; i<r; ++i))
    do
        s="${s}${c}"
    done
    echo $s
}

path=$(dw get path)
path=${path%\'}
path=${path##*\'}

[[ -e $path/prodlist ]] && rm -f $(sed -e "s|^|${path}/|" $path/prodlist)

rm -f $path/prodlist

for assertion in $(dw get assert | awk '{sub(/:/, "", $1); print $1}')
do
    ${DWAVE_HOME}/bin/dw-special.bash \
	$path/$assertion \
	$path/$assertion.coefficient \
	> $path/$assertion.posiform ||

    ${DWAVE_HOME}/bin/dw-con-search.bash \
	$path/$assertion > $path/$assertion.posiform ||

    if [[ -n $verbose ]]
    then
	${DWAVE_HOME}/bin/dw-deqo.bash $assertion > $path/$assertion.deqo

	flatten < $path/$assertion.deqo > $path/$assertion.posiform
    else
	${DWAVE_HOME}/bin/dw-deqo.bash $assertion | flatten > $path/$assertion.posiform
    fi

    validate < $path/$assertion.posiform > $path/$assertion.posiform.CHECK
done
