#!/bin/bash

# Creates and executes a QMI (Quantum Machine Instruction)
# directly by specifying qubit weights and coupler strengths

# Exit if not being run via Bash shell:
if [ ! -n "$BASH" ] ; then 
    echo Please run this script $0 with bash; exit 1; 
fi

# Get a connection to the local simulator:
dw set connection laptop

# Get a 128-qubit sampling solver from the simulator:
dw set solver c4-sw_sample

# Make new workspace directory (examples) if it does not exist:
dw mkdir -p examples

# Change to new workspace directory:
dw cd examples

# Initialize the workspace:
dw init

# Create a QMI which makes two qubits align:
dw set same.qmi <<EOF
    Q0000 <== 5
    Q0004 <== 5
    C0000 <== -10
EOF

# Execute the QMI:
dw exec num_reads=1000 same.qmi

# Display the solution:
dw get same.sol
