#!/bin/bash

# Qbsolv is an alternate back-end in dw.  In this example, use
# the constraint compiler to build a 3x3 permutation matrix and
# generate a QUBO.  Rather then embedding the QUBO (via "dw embed")
# use "dw qbsolv" to prepare the QUBO for execution with qbsolv.

# Exit if not being run via Bash shell
if [ ! -n "$BASH" ] ; then 
    echo Please run this script $0 with bash; exit 1; 
fi

# Get a connection to the local simulator:
dw set connection laptop

# Get a 128-qubit sampling solver from the simulator:
dw set solver c4-sw_sample

# Make new workspace directory (examples) if it does not exist:
dw mkdir -p examples

# Change to new workspace directory:
dw cd examples

# Initialize workspace:
dw init

# Define nine variables forming a 3x3 matrix:
for ((r=1; r<=3; ++r))
do
    for ((c=1; c<=3; ++c))
    do
	dw set variable x${r}${c}
    done
done

# For each row assert that the variables add to 1:
for ((r=1; r<=3; ++r))
do
    dw set assert "R3_one_of_n(x${r}1,x${r}2,x${r}3)"
done

# For each column assert that the variables add to 1:
for ((c=1; c<=3; ++c))
do
    dw set assert "R3_one_of_n(x1${c},x2${c},x3${c})"
done

# Compile the constraints to create the QUBO:
dw compile > J.q

# Prepare the QUBO for execution with qbsolv:
dw qbsolv J.q

# Prepare for execution with an empty bind, since we have no parameters:
dw bind empty

# Execute the qbsolv job:
dw exec default.qbi

# Validate the answer:
dw val default.qbo
